import unittest
import numpy as np

from cxroots.IterativeMethods import secant
from scipy import pi, cos, sin

class TestSecant(unittest.TestCase):
	def test_secant(self):
		# example from Table 2.5 of "Numerical Analysis" by Richard L. Burden, J. Douglas Faires
		f  = lambda x: cos(x)-x
		df = lambda x: -sin(x)-1

		iterations = []
		callback = lambda x, dx, y, iteration: iterations.append(x)
		x, err = secant(0.5, pi/4, f, callback=callback)
		iterations.append(x)
		
		correct_iterations = [0.7363841388,
			  				  0.7390581392,
			  				  0.7390851493,
			  				  0.7390851332,
			  				  0.7390851332]

		np.testing.assert_almost_equal(iterations, correct_iterations, decimal=10)

if __name__ == '__main__':
	unittest.main()



#!/usr/bin/env python
import os
import sys
import shutil
import unittest
from distutils.core import setup, Command
from numpy.distutils.misc_util import get_numpy_include_dirs

packages = ['cxroots', 'cxroots.tests']

# get the version
exec(open('cxroots/version.py').read())

# read the README_pip.rst
try:
    with open('README_pip.rst') as file:
        long_description = file.read()
except:
    long_description = None

# create test commmand
class TestCommand(Command):
    # See: https://justin.abrah.ms/python/setuppy_distutils_testing.html by Justin Abrahms
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import sys, subprocess

        raise SystemExit(
            subprocess.call([sys.executable,
                             '-m',
                             'unittest',
                             'discover',
                             '-v',
                             'cxroots/tests']))

setup(
    name = 'cxroots',
    version = __version__,
    description = 'Find all the roots (zeros) of a complex analytic function within a given contour in the complex plane.',
    long_description = long_description,
    author = 'Robert Parini',
    author_email = 'rp910@york.ac.uk',
    url = 'https://rparini.github.io/cxroots/',
    license = 'BSD',
    packages = packages,
    platforms = ['all'],
    install_requires = ['numpy', 'scipy', 'docrep', 'mpmath'],
    keywords='roots zeros complex analytic functions',
    classifiers=[
	    'Development Status :: 4 - Beta',
	    'Intended Audience :: Science/Research',
	    'Topic :: Scientific/Engineering :: Mathematics',
	    'License :: OSI Approved :: BSD License',
	    'Programming Language :: Python :: 2.7',
	    'Programming Language :: Python :: 3',
	],
    cmdclass={
        'test': TestCommand,
    }
)



import unittest
import numpy as np

from cxroots.IterativeMethods import muller
from scipy import pi, cos, sin

class TestSecant(unittest.TestCase):
	def test_secant(self):
		# example from Table 2.5 of "Numerical Analysis" by Richard L. Burden, J. Douglas Faires
		f  = lambda x: cos(x)-x**2 + x**3 * 1j

		iterations = []
		callback = lambda x, dx, y, iteration: iterations.append(x)
		x, err = muller(0.5, pi/4, 0.6, f)

		root = 0.7296100078977741539157356847 + 0.1570923181734581909733787621j
		self.assertAlmostEqual(x, root)

if __name__ == '__main__':
	unittest.main()



import unittest
import numpy as np
from scipy import pi, sqrt, exp, sin, cos

from cxroots import Circle, Rectangle
from cxroots.tests.ApproxEqual import roots_approx_equal

### XXX: Need some way to distinguish clusters of roots

class TestCluster_1(unittest.TestCase):
	def setUp(self):
		self.roots = roots = [3, 3.00001, 3.00002, 8, 8.00002, 8+0.00001j]
		self.multiplicities = [1,1,1,1,1,1]

		self.C = Rectangle([2,9], [-1,1])
		self.f = lambda z: np.prod([z-r for r in roots])
		self.df = lambda z: np.sum([np.prod([z-r for r in np.delete(roots,i)]) for i in range(len(roots))])

	def test_rootfinding_df(self):
		roots_approx_equal(self.C.roots(self.f, self.df), (self.roots, self.multiplicities), decimal=12)

	# def test_rootfinding_f(self):
	# 	roots_approx_equal(self.C.roots(self.f), (self.roots, self.multiplicities), decimal=12)

# class TestCluster_2(unittest.TestCase):
# 	def setUp(self):
# 		self.roots = roots = [3, 3.00001, 3.00002, 8, 8.00002, 8+0.00001j]
# 		self.multiplicities = [1,1,1,1,1,1]

# 		self.C = Circle(0, 8.5)
# 		self.f = lambda z: np.prod([z-r for r in roots])
# 		self.df = lambda z: np.sum([np.prod([z-r for r in np.delete(roots,i)]) for i in range(len(roots))])

# 	def test_rootfinding_df(self):
# 		roots_approx_equal(self.C.roots(self.f, self.df), (self.roots, self.multiplicities), decimal=12)

# 	def test_rootfinding_f(self):
# 		roots_approx_equal(self.C.roots(self.f), (self.roots, self.multiplicities), decimal=12)


if __name__ == '__main__':
	# unittest.main(verbosity=3)

	roots = [3, 3.00001, 3.00002, 8, 8.00002, 8+0.00001j]
	multiplicities = [1,1,1,1,1,1]

	f = lambda z: np.prod([z-r for r in roots])
	df = lambda z: np.sum([np.prod([z-r for r in np.delete(roots,i)]) for i in range(len(roots))])

	C = Circle(0, 8.5)
	# C = Rectangle([2,9], [-1,1])

	# C.approximate_roots(f, df, verbose=True)
	C.demo_roots(f, df, verbose=True)
